
"""PgQ event container.
"""

# Event status codes
EV_RETRY = 0
EV_DONE = 1
EV_FAILED = 2

class Event:
    """Event data for consumers.
    
    Consumer is supposed to tag them after processing.
    If not, events will stay in retry queue.
    """
    def __init__(self, queue_name, event_data_form_db):
        self.status = EV_RETRY
        self.retry_time = 60
        self.fail_reason = "Buggy consumer"

        event_data = {'event_id':-1, 'retry_count':0}
        event_data.update(event_data_form_db)

        self.queue_name = queue_name
        self.producer = event_data['producer']
        self.ref_id = event_data['ref_id']
        self.payload = event_data['payload']
        self.id = event_data['event_id']
        self.creation_date = event_data['creation_date']
        self.event_txid = event_data['event_txid']
        self.retry_count = event_data['retry_count']

    def tag_done(self):
        self.status = EV_DONE

    def tag_retry(self, retry_time = 60):
        self.status = EV_RETRY
        self.retry_time = retry_time

    def tag_failed(self, reason):
        self.status = EV_FAILED
        self.fail_reason = reason

