
create or replace function pgq.batch_event_tables(x_batch_id bigint)
returns setof text as $$
--
-- return table names to be searched for events
--
declare
    r_start               bigint;
    r_end                 bigint;
    nr                    smallint;
    tbl                   text;
    x_tblname             text;
    x_active              smallint;
    x_numtables           smallint;
    step1                 bigint;
    step2                 bigint;
    use_prev              integer;
    use_next              integer;
    q                     record;
begin
    select get_snapshot_xmin(prev.snapshot) as tx_min, -- absolute minimum
           get_snapshot_xmax(cur.snapshot) as tx_max, -- absolute maximum
           cf.table_name, cf.number_of_event_tables,
           cf.active_event_table, cf.last_switch_step1, cf.last_switch_step2
        into q
        from pgq.tick cur, pgq.tick prev, pgq.active_batch b,
             pgq.queue_config cf
        where cur.tick_id = b.tick_id
          and cur.queue_id = b.queue_id
          and prev.tick_id = b.prev_tick_id
          and prev.queue_id = b.queue_id
          and b.batch_id = x_batch_id
          and cf.queue_id = b.queue_id;
    if not found then
        raise exception 'Cannot find data for batch %', x_batch_id;
    end if;

    -- if its definitely not in one or other, look into both
    if q.tx_max < q.last_switch_step1 then
        use_prev := 1;
        use_next := 0;
    elsif q.last_switch_step2 is not null
      and (q.tx_min > q.last_switch_step2)
    then
        use_prev := 0;
        use_next := 1;
    else
        use_prev := 1;
        use_next := 1;
    end if;

    if use_prev then
        nr := q.active_event_table - 1;
        if nr < 0 then
            nr := q.number_of_event_tables - 1;
        end if;
        tbl := q.table_name || '_' || nr;
        return next tbl;
    end if;

    if use_next then
        tbl := q.table_name || '_' || q.active_event_table;
        return next tbl;
    end if;

    return;
end;
$$ language plpgsql; -- no perms needed


