create or replace function pgq.batch_retry(
    x_batch_id bigint,
    x_retry_seconds integer)
returns integer as $$
declare
    new_retry  timestamptz;
    bt         record;
    cnt        integer;
begin
    new_retry := current_timestamp + ((x_retry_seconds || ' seconds')::interval);

    select * from pgq.active_batch into bt
        where batch_id = x_batch_id;

    insert into pgq.retry_event (queue_id, consumer_id, batch_id, event_id,
         retry_count, retry_after_date,
         creation_date, ref_id, payload, producer) 
    select distinct b.queue_id, b.consumer_id, b.batch_id, ev.event_id,
        coalesce(ev.retry_count, 0) + 1, new_retry, 
         ev.creation_date, ev.ref_id, ev.payload, ev.producer 
      from pgq.active_batch b, pgq.fetch_batch_events(x_batch_id) ev
        left join pgq.retry_event re
          on (re.event_id = ev.event_id
              and re.consumer_id = bt.consumer_id
              and re.queue_id = bt.queue_id)
        where b.batch_id = x_batch_id
          and re.event_id is null;

    GET DIAGNOSTICS cnt = ROW_COUNT;

    return cnt;
end;
$$ language plpgsql security definer;

