create or replace function pgq.drop_queue(x_queue_name text)
returns integer as $$
declare
    tblname  text;
    cf record;
    num integer;
begin
    -- check ares
    if x_queue_name is null then
        raise exception 'Invalid NULL value';
    end if;

    -- check if exists
    select * from pgq.queue_config
        where queue_name = x_queue_name
        into cf;
    if not found then
        raise exception 'No such event queue';
    end if;

    -- check if no consumers
    select count(*) into num from pgq.high_watermark
        where queue_id = cf.queue_id;
    if num > 0 then
        raise exception 'cannot drop queue, consumers still attached';
    end if;

    -- drop data tables
    for i in 0 .. (cf.number_of_event_tables - 1) loop
        tblname := cf.table_name || '_' || i;
        execute 'DROP TABLE ' || tblname;
    end loop;
    execute 'DROP TABLE ' || cf.table_name;

    -- delete ticks
    delete from pgq.tick where queue_id = cf.queue_id;

    -- delete retry and failed events
    delete from pgq.retry_event where queue_id = cf.queue_id;
    delete from pgq.failed_event where queue_id = cf.queue_id;

    -- drop seqs
    -- FIXME: any checks needed here?
    execute 'DROP SEQUENCE ' || cf.tick_seq_name;
    execute 'DROP SEQUENCE ' || cf.event_seq_name;

    -- delete event
    delete from pgq.queue_config
        where queue_name = x_queue_name;

    return 1;
end;
$$ language plpgsql security definer;

