
/*
create type pgq.ret_failed_event_data as (
    event_id bigint,
    ref_id text,
    payload text,
    producer text,
    creation_date timestamptz,
    retry_count integer,
    failure_date timestamptz,
    failure_reason text
);
*/

create or replace function pgq.failed_event_list(x_queue_name text, x_consumer_name text)
returns setof pgq.ret_failed_event_data as $$ 
declare 
    rec pgq.ret_failed_event_data%rowtype; 
begin 
    for rec in
        select f.event_id, f.ref_id, f.payload, f.producer,
               f.creation_date, f.retry_count,
               f.failure_date, f.failure_reason
          from pgq.failed_event f, pgq.consumer c,
               pgq.queue_config q
         where q.queue_name = x_queue_name
           and c.consumer_name = x_consumer_name
           and f.queue_id = q.queue_id
           and f.consumer_id = c.consumer_id
        order by failure_date
    loop
        return next rec; 
    end loop; 
    return;
end; 
$$ language plpgsql security definer;

create or replace function pgq.failed_event_list(
    x_queue_name text,
    x_consumer_name text,
    x_count integer,
    x_offset integer)
returns setof pgq.ret_failed_event_data as $$ 
declare 
    rec pgq.ret_failed_event_data%rowtype; 
begin 
    for rec in
        select f.event_id, f.ref_id, f.payload, f.producer,
               f.creation_date, f.retry_count,
               f.failure_date, f.failure_reason
          from pgq.failed_event f, pgq.consumer c,
               pgq.queue_config q
         where q.queue_name = x_queue_name
           and c.consumer_name = x_consumer_name
           and f.queue_id = q.queue_id
           and f.consumer_id = c.consumer_id
        order by failure_date
        limit x_count
        offset x_offset
    loop
        return next rec; 
    end loop; 
    return;
end; 
$$ language plpgsql security definer;

create or replace function pgq.failed_event_count(
    x_queue_name text,
    x_consumer_name text)
returns integer as $$ 
declare 
    ret integer;
begin 
    select count(1) into ret
      from pgq.failed_event f, pgq.consumer c,
           pgq.queue_config q
     where q.queue_name = x_queue_name
       and c.consumer_name = x_consumer_name
       and f.queue_id = q.queue_id
       and f.consumer_id = c.consumer_id;
    return ret;
end; 
$$ language plpgsql security definer;

create or replace function pgq.failed_event_delete(
    x_queue_name text,
    x_consumer_name text,
    x_event_id bigint)
returns integer as $$ 
declare 
    x_queue_id    integer;
    x_consumer_id integer;
begin 
    select q.queue_id, c.consumer_id into x_queue_id, x_consumer_id
      from pgq.consumer c, pgq.queue_config q
     where q.queue_name = x_queue_name
       and c.consumer_name = x_consumer_name;
    if not found then
        raise exception 'no such queue/consumer';
    end if;

    delete from pgq.failed_event
     where queue_id = x_queue_id
       and consumer_id = x_consumer_id
       and event_id = x_event_id;
    if not found then
        raise exception 'event not found';
    end if;

    return 1;
end; 
$$ language plpgsql security definer;

create or replace function pgq.failed_event_retry(
    x_queue_name text,
    x_consumer_name text,
    x_event_id bigint)
returns bigint as $$ 
declare 
    ret           bigint;
begin 
    select pgq.insert_event_raw(
             f.producer, x_queue_name, f.ref_id, f.payload,
             f.event_id, f.creation_date, c.consumer_id, f.retry_count)
      into ret
      from pgq.failed_event f, pgq.consumer c, pgq.queue_config q
     where q.queue_name = x_queue_name
       and c.consumer_name = x_consumer_name
       and f.queue_id = q.queue_id
       and f.consumer_id = c.consumer_id
       and f.event_id = x_event_id;
    if not found then
        raise exception 'event not found';
    end if;

    perform pgq.failed_event_delete(x_queue_name, x_consumer_name, x_event_id);

    return ret;
end; 
$$ language plpgsql security definer;


