create or replace function pgq.ticker(
    i_queue_id integer,
    i_tick_id bigint)
returns bigint as $$
-- insert a tick for queue, with specific ID
begin
    insert into pgq.tick (queue_id, tick_id)
        values (i_queue_id, i_tick_id);
    return i_tick_id;
end;
$$ language plpgsql security definer; -- unsure about access

create or replace function pgq.ticker(
    i_queue_name text,
    i_tick_id bigint)
returns bigint as $$
-- insert a tick for queue, with specific ID
begin
    insert into pgq.tick (queue_id, tick_id)
    select queue_id, i_tick_id
        from pgq.queue_config
        where queue_name = i_queue_name
          and external_ticker;
    if not found then
        raise exception 'queue not found';
    end if;
    return i_tick_id;
end;
$$ language plpgsql security definer; -- unsure about access

create or replace function pgq.ticker(
    i_queue_id integer)
returns bigint as $$
-- insert a tick for queue, taking next ID from seq
declare
    res bigint;
    ext boolean;
    seq text;
begin
    select tick_seq_name, external_ticker into seq, ext
        from pgq.queue_config
        where queue_id = i_queue_id;
    if not found then
        raise exception 'no such queue';
    end if;
    if ext then
        raise exception 'This queue has external tick source.';
    end if;

    select pgq.ticker(i_queue_id, nextval(seq)) into res;

    return res;
end;
$$ language plpgsql security definer; -- unsure about access

create or replace function pgq.ticker(i_queue_name text) returns bigint as $$
declare
    res bigint;
begin
    select pgq.ticker(queue_id) into res
        from pgq.queue_config
        where queue_name = i_queue_name;
    if not found then
        raise exception 'no such queue: %', i_queue_name;
    end if;
    return res;
end;
$$ language plpgsql security definer;

create or replace function pgq.ticker() returns bigint as $$
declare
    res bigint;
begin
    select count(pgq.ticker(queue_id)) into res
        from pgq.queue_config
        where not external_ticker;
    return res;
end;
$$ language plpgsql security definer;

